type op = Add | Sub | Mult | Div | Equal | Neq | Less | Leq | Greater | Geq | Combine

type objectexpr =          (* Object Expressions *)
  Literal of string    
  | Str of string      
  | Access of string * string   
  | Binop of objectexpr * op * objectexpr
  | AssignToObject of string * string * objectexpr
  | Noexpr
    
type expr =           (* Expressions *)
  Id of string         
  | Assign of string * string * objectexpr    
  | Call of string * expr list     

type stmt =
    Block of stmt list
  | Expr of expr
  | If of objectexpr * stmt * stmt
  | For of objectexpr * objectexpr * objectexpr * stmt
  | ToScreen of objectexpr
    
type p_decl = {
  key    : string;
  value  : string;         
}

type v_decl = {
  vartype     : string;         
  varname     : string;         
  varparams   : p_decl list;        
}  
  
type func_decl = {
    fname : string;
    formals : string list;
    locals : v_decl list;
    body : stmt list;
  }

  type program = v_decl list * func_decl list
